breed [drones drone]
breed [stations stattion]

patches-own[
  fire ;; 0 = no fire, 1 = fire just started, 2 = fire lvl 2; 3 = fire lvl 3; 4 = fire lvl 4; 5 = fire at max level
  grow ;;after 11 ticks the fire grows
  ;; when the fire grows, the patches around it turn red and have fire = 1

]

drones-own [
  shortVision
  longVision
  scoutFireX
  scoutFireY
]

globals [
  battery ;; the battery of the drone
  xFire ;; x-coordinate of the fire, used when the fire is increased
  yFire ;; y-coordinate of the fire, used when the fire is increased
  xDestination ;; the x-coordinate of the location sent by the operator at the base
  yDestination ;; the y-coordinate of the lcoaction sent by the operator at the base
  randXFire ;; x-coordinate of the first patch that is on fire
  randYFire ;; y-coordinate of the first patch that is on fire
  destinations ;; list that will contain lists of the locations sent by the operator
  location ;; list containing the x,y-coordinates of the location sent by the operator, as well as the distance between that location and drone, those lcoations will be stored in list destinations
  countFire ;; counts the amount of fire patches that the short range camera detects
  countForest ;; counts the amount of forest patches that the short range camera detects
  countDirt ;; counts the amount of dirt patches that the short range camera detects
  countSand ;; counts the amount of sand patches that the short range camera detects
  destination ;; used to see if the drone arrived at the location indicated by the operator
  dist ;; used to put the location chosen by the operator in the destinations list
  wind ;; can have value 0 (no wind), 1(from west), 2(from east), 3(from north), 4(from south)
  vx1 ;; represents the right border (when the drone reached the rightmost points and cannot go further to the right, used to change the direction of the drone when scouting)
  vx0 ;; represents the left border (when the drone reached the leftmost points and cannot go further to the left, used to change the direction of the drone when scouting)
  vy1 ;; represents the top border (when the drone reached the upmost points and cannot go further to the top, used to change the direction of the drone when scouting)
  vy0 ;; represents the bottom border (when the drone reached the downmost points and cannot go further to the bottom, used to change the direction of the drone when scouting)
  diameter ;; the size of the diameter of the long range camera, used so the drone scouts the area efficiently
  xMax ;; max x-coordinate of the area, used for scouting
  yMax ;; max y-coordinate of the area, used for scouting
  xMin ;; min x-coordinate of the area, used for scouting
  yMin ;; min y-coordinate of the area, used for scouting
  scout? ;; checks if the drone should scout or has locations to visit
  dir ;; used for scouting, indicates if the drone should go to the left (value 0) or to the right (value 1)
  foundFire ;; counts how many runs of the simulations have been ended by the drone finding the fire in time, as displayed on the UI
  fireTooBig ;; counts how many runs of the simulations have been ended by the fire getting too big, as displayed on the UI
  avgBatteryLife ;; computes the average battery life of the drone after each run of the simulation where the drone found the fire, as displayed on the UI
  dataCollected ;; boolean guard that determines whether the simulation is allowed to reset itself. Should only reset if a run has ended and the three variables defined above this one are updated
  maxFireSize ;; defines the limit for how many patches may be fire patches

  ffc ;; the coordinates of the found fire, used when testing the efficency of the simulation
  ftbc ;; the coordinates of the fires that became too big and were not found by the drone in time, used when testing the efficency of the simulation
  coord ;; used when showing the coordinates above, used when testing the efficency of the simulation
]

to setup
  clear-all
  import-pcolors "zoomedIn.png"
  setup-patches
  setup-turtles
  setup-turtles1
  set xDestination 600
  set yDestination 600
  set destinations []
  set location []
  set vx0 0
  set vx1 0
  set vy0 0
  set vy1 0
  set diameter 240
  set yMax 370
  set xMax 500
  set yMin -370
  set xMin -500
  set wind random 5
  set scout? 1
  set dir 0
  set dataCollected false
  set maxFireSize 300
  set ffc []
  set coord []
  set ftbc []
  reset-ticks
end

;; move or expand everything by one tick, then check whether the fire is too big and if the simulation needs to restart
to go
  move-turtles
  tick
  checkFireSize
  restartSimulation
end

to setup-turtles
  create-drones 1 [
    set color white
    set size 3
    set battery droneBattery
    setxy xCoordBase yCoordBase
    facexy xcor ycor + 1
    ;; sets the range and the angle of vision for the short-range camera and the long-range camera
    set shortVision patch-set patches in-cone 67 360
    ask shortVision [set pcolor pcolor + 1.5]
    set LongVision patch-set patches in-cone 250 360
    ask longVision [set pcolor pcolor + 1.5]
    ;; Default starting values: These coordinates are out of bounds and can therefore safely be assigned
    set scoutFireX 600
    set scoutFireY 600
  ]

end

;; the base is created
to setup-turtles1
  create-stations 1 [
    set color pink
    set size 8
    setxy xCoordBase yCoordBase
  ]
end


to setup-patches
  if restart [import-pcolors "zoomedIn.png"]

  set randXFire 450 - (random 900)
  set randYFire 300 - (random 660)


  ask patches [
    set fire 0
    set grow 0
  ]
  ask patch randXFire randYFire [
    print pxcor
    print pycor
    set pcolor red
    set fire -10
  ]
end

;; if restart is enabled in the UI and a resolution has ended, then reset all values to restart the simulation
to restartSimulation
  if restart and dataCollected [
    set dataCollected false
    if foundFire != 0 [
      set avgBatteryLife (avgBatteryLife * (foundFire - 1) + battery) / foundFire
    ]
    ask drones [
      ;; Reset pcolors to remove the vision
      ask shortVision [set pcolor pcolor - 1.5]
      set shortVision patch-set patches in-cone 67 360
      ask longVision [set pcolor pcolor - 1.5]
      set longVision patch-set patches in-cone 250 360
      die
    ]
    set destination []
    set location []
    set destinations []
    setup-patches
    setup-turtles
    set wind random 5
    set vx0 0
    set vx1 0
    set vy0 0
    set vy1 0
    set dir 0

  ]
end


to move-turtles
  move-drones
  ask drones[
    set battery battery - 1
    check-battery
  ]
  check-fire
end


to move-drones
  set countFire 0
  set countForest 0
  set countDirt 0
  set countSand 0
  set dist 0


  ask drones [
    let fireX xcor
    let fireY ycor

    ;; Remove highlight from the previous vision cones
    ask shortVision [set pcolor pcolor - 1.5]
    set shortVision patch-set patches in-cone 67 360
    ask longVision [set pcolor pcolor - 1.5]
    set longVision patch-set patches in-cone 250 360

    ;; long range, thermal camera detection. Only looks for red patches
    ask longVision [
      if pcolor = red or pcolor = red - 1.5 or pcolor = red + 1.5 [
        set fireX pxcor
        set fireY pycor
      ]
    ]
    ;; delete all waypoints and go check the anomaly asap
    if fireX != xcor or fireY != ycor [
      set destinations [] ;; remove all waypoints
      set location [] ;; reset location in case it was not already empty

      ;; put coordinates into location, then put these values into destinations
      set location lput fireX location
      set location lput fireY location
      set location lput dist location
      set destinations lput location destinations
      set location []
      set xDestination 600 ;; the x-coordinate is reset, will get a new value when the operator chooses another location on the map
      set yDestination 600 ;; the y-coordinate is reset, will get a new value when the operator chooses another location on the map
      set scoutFireX fireX
      set scoutFireY fireY
    ]

    (ifelse
      ;; if an anomaly has been found, then start moving in that direction
      scoutFireX = fireX and scoutFireY = fireY [
        facexy scoutFireX scoutFireY

        ;; use short range camera to get the specific coordinates of the anomaly, ignore current terrain to get to the place of interest asap
        ask shortVision [
        if  pcolor = red or pcolor = red - 1.5 or pcolor = red - 3
            [set countFire countFire + 1]
        ]
      ]
      ;; short range, broad camera for fire and terrain detection. Analyses also terrain type if there is nothing interesting is detected
      [ask shortVision [
        (ifelse
          pcolor = red or pcolor = red - 1.5 or pcolor = red - 3
            [set countFire countFire + 1]
          pcolor <= 1.7 or pcolor >= 70
            [set countForest countForest + 1]
          pcolor <= 2.5
            [set countDirt countDirt + 1]
          [set countSand countSand + 1])
        ]

        ;; when the operator chose a location on the map
        ifelse xDestination != 600 and yDestination != 600 [
          set scout? 0 ;; the drone does not scout anymore and needs to go the location indicated by the operator
          ifelse empty? destinations [
            ask drones [
              set dist ((xDestination - xcor) ^ 2 + (yDestination - ycor) ^ 2) ^ 0.5 ;;using pythagoras to find the shortest distance to the destination (Euclidean distance)
            ]
            set location lput xDestination location ;; x-coordinate of the location is put in a list
            set location lput yDestination location ;; y-coordinate of the location is added to the list
            set location lput dist location ;; the distance to the location is added to the list
            set destinations lput location destinations ;; the list is added to the destinations list
            set xDestination 600 ;; the x-coordinate is reset, will get a new value when the operator chooses another location on the map
            set yDestination 600 ;; the y-coordinate is reset, will get a new value when the operator chooses another location on the map
            set location [] ;; the list is rest so it can be used again for the next location

          ][
            ask drones [
              set dist ((xDestination - xcor) ^ 2 + (yDestination - ycor) ^ 2) ^ 0.5 ;;using pythagoras to find the shortest distance to the destination (Euclidean distance)
            ]
            set location lput xDestination location
            set location lput yDestination location
            set location lput dist location
            if location != last destinations [
              set destinations lput location destinations
            ]
            set xDestination 600
            set yDestination 600
            set location []
          ]
          update-dist ;; the distances from the drone to the locations are updated because the drone has moved
          sort-list ;; the destinations with all the locations sent by the operator is sorted so the drone visits the closest location first
          move
        ] [
          ifelse empty? destinations [ ;; there are no locations left to visit so the drone gets back to scouting
            ifelse scout? = 1 [
              ask drones [
                scout3
              ]
            ][
               ask drones[
               if abs(xcor - xCoordBase) < 1 and abs(ycor - yCoordBase) < 1 [
                  set scout? 1
                  set heading 0 ;;IT WAS 270 BEFORE IDK WHY
                ]
              ]
            ]
          ][
            move
          ]

        ]
        print destinations
    ])

    ;;Set highlights for camera's in the new position of the drone
    ask longVision [set pcolor pcolor + 1.5]
    ask shortVision [set pcolor pcolor + 1.5]

    ;; if the short-range camera detects a fire, then end the simulation or restart the simulation based on restart in the UI
    if countFire > 0 [
      print "Sending fire coordinates"
      ;facexy xcor ycor + 1
      (ifelse restart [
        set foundFire foundFire + 1
        set coord lput randXFire coord
        set coord lput randYFire coord
        set coord lput wind coord
        set ffc lput coord ffc
        set coord []
        set dataCollected true
        ]
        [user-message (word "Fire detected")])
    ]
  ]
end


to move
  ifelse empty? destinations [ ;; if there are no locations to visit, the drone scouts
    scout3
  ][ ;; else, the drone checks if it arrived at the first location on from the list or not
    if destination != first destinations [set destination first destinations

    print destination]
    ask drones [ ;; check if drone arrived at destination
      facexy first destination item 1 destination
      if abs(first destination - xcor) < 1 and abs(item 1 destination - ycor) < 1 [
        set destinations but-first destinations
        set xDestination 600
        set yDestination 600
        set location []

        ifelse empty? destinations [
          facexy xCoordBase yCoordBase
        ][
          update-dist
          sort-list
        ]
      ]

    ]
  ]
end

;; the locations in the list destinations are sorted in ascending by the distance from the drone to the respective locations
to sort-list
  set destinations sort-by [[list1 list2] -> last list1 < last list2] destinations
end

;; the distance from the drone to the locations is updated for all locations since the drone changed its current position
to update-dist
  let d destinations
  set destinations []
  let len 0
  let l length d
  while [len < l][
    let loc first d
    ask drones[
      let x first loc
      let y item 1 loc
      set dist ((x - xcor) ^ 2 + (y - ycor) ^ 2) ^ 0.5 ;;using euclidean distance
    ]
    set loc but-last loc
    set loc lput dist loc
    set destinations lput loc destinations
    set d but-first d
    set len len + 1
  ]
end

;; checks if the drone has enough battery left or not
to check-battery
  let distb ((xCoordBase - xcor) ^ 2 + (yCoordBase - ycor) ^ 2) ^ 0.5 ;; the distance from the drone to the base

  let distToStation distb
  let x xCoordBase
  let y yCoordBase

  if battery < distToStation - 10 [ ;; if the drone has little battery left, it goes to the base to charge
    facexy x y
  ]

  ;;checks if it arrived at the station
  if abs(xcor - x) < 1 and abs(ycor - y) < 1 [
    set battery droneBattery ;; the battery is changed
    set destination []

    ;; the time in which the battery was changed was simulated by increasing the fire 4 times
    check-fire
    check-fire
    check-fire
    check-fire
    set heading 0
  ]

  forward 1
end

;; the scouting algorithm
to scout3
  if ycor > yMax - diameter [
    ifelse xcor < xMin + diameter [
      ifelse vx1 = 0 [
        set heading 90
        set dir 1
        set vx1 vx1 + 1
      ][
        ifelse vx0 > 0 [
          set heading 90
          set vx1 vx1 + 1
        ][
          set heading 180
        ]

      ]
    ][
      ifelse xcor > xMax - diameter [
        ifelse vx1 = 0 [
          set heading 90
          set dir 0
          set vx1 vx1 + 1
      ][
          set vx0 0
          set heading 180
        ]
      ][
        ifelse dir = 1 [
          set heading 90
        ][
          set heading 270
        ]
        set vx1 vx1 + 1
      ]
    ]
  ]

  if ycor < yMin + diameter [
    ifelse xcor < xMin + diameter [
      ifelse vx0 = 0 [
        set heading 90
        set dir 1
        set vx0 vx0 + 1
      ][
        ifelse vx1 > 0 [
          set heading 270
          set dir 0
          set vx0 vx0 + 1
        ][
          set heading 0
        ]
      ]
    ][
      ifelse xcor > xMax - diameter [
        ifelse vx0 = 0 [
          set heading 270
          set dir 0
          set vx0 vx0 + 1
      ][
          set vx0 0
          set heading 180
        ]
      ][
        ifelse dir = 1 [
          set heading 90
        ][
          set heading 270
        ]
        set vx0 vx0 + 1
      ]
    ]
  ]

   if vx1 = diameter [
    set heading 180
    set vx1 0
    set vx0 0
  ]
  if vx0 = diameter [
    set heading 0
    set vx0 0
    set vx1 0
  ]
end


;; the fire is expanded as in real time
to check-fire
  ask patches with [pcolor = red or pcolor = red - 1.5 or pcolor = red + 1.5] [
    ifelse grow < 11 and fire != 5 [
      set grow grow + 1
    ][
      if fire < 5 [
        set fire fire + 1
        set grow 0
      ]
    ]
  ]
  expand-fire
end


to expand-fire
  expand
  expand-up
  expand-down
  expand-left
  expand-right
end

;; how the fire expands if there is no wind
to expand
  if wind = 0 [
    ask patches with [fire = 4] [
      set grow 0
      set xFire pxcor + 1
      set yFire pycor
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]
    ]

    ask patches with [fire = 4] [
      set grow 0
      set xFire pxcor - 1
      set yFire pycor
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]
    ]
    ask patches with [fire = 4] [
      set grow 0
      set xFire pxcor
      set yFire pycor + 1
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]
    ]
    ask patches with [fire = 4] [
      set grow 0
      set xFire pxcor
      set yFire pycor - 1
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]
    ]
  ]
end

;; how the fire expands if the wind is coming from west
to expand-right
  if wind = 1 [
    ask patches with [fire = 4] [
      set xFire pxcor + 1
      set yFire pycor
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]

      set xFire pxcor + 1
      set yFire pycor + 1
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]

      set xFire pxcor + 1
      set yFire pycor - 1
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]
    ]
  ]
end


;; how the fire expands if the wind is coming from east
to expand-left
  if wind = 2 [
    ask patches with [fire = 4] [
      set xFire pxcor - 1
      set yFire pycor
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]
      set xFire pxcor - 1
      set yFire pycor + 1
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]

      set xFire pxcor - 1
      set yFire pycor - 1
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]
    ]
  ]
end


;; how the fire expands if the wind is coming from south
to expand-up
  if wind = 4 [
    ask patches with [fire = 4] [
      set xFire pxcor
      set yFire pycor + 1
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]

      set xFire pxcor + 1
      set yFire pycor + 1
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]

      set xFire pxcor - 1
      set yFire pycor + 1
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]
    ]
  ]
end


;; how the fire expands if the wind is coming from north
to expand-down
  if wind = 3 [
    ask patches with [fire = 4] [
      set xFire pxcor
      set yFire pycor - 1
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]
      set xFire pxcor + 1
      set yFire pycor - 1
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]

      set xFire pxcor - 1
      set yFire pycor - 1
      ask patch xFire yFire [
        if pcolor != red [
          set pcolor red
          set fire 1
        ]
      ]
    ]
  ]

end

;; checks if the fire is larger or equal than maxFireSize
;; then it either terminates the simulation or prepares the simulation for the next run, depending on the boolean restart in the UI
to checkFireSize
  let fireSize 0
  ask patches with [pcolor = red or pcolor = red + 1.5 or pcolor = red - 1.5] [
    set fireSize fireSize + 1
  ]
  if fireSize >= maxFireSize [
    if restart = false [user-message (word "Fire too large")]
    set fireTooBig fireTooBig + 1
    set dataCollected true
    set coord lput randXFire coord
    set coord lput randYFire coord
    set coord lput wind coord
    set ftbc lput coord ftbc
    set coord []
  ]
end
@#$#@#$#@
GRAPHICS-WINDOW
251
11
1260
761
-1
-1
1.0
1
10
1
1
1
0
0
0
1
-500
500
-370
370
1
1
1
ticks
30.0

BUTTON
10
44
73
77
setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
79
45
142
78
go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
6
89
178
122
droneBattery
droneBattery
0
10800
10800.0
1
1
NIL
HORIZONTAL

INPUTBOX
1384
85
1456
145
XCoordBase
0.0
1
0
Number

PLOT
1282
257
1482
407
plot 1
time
battery
0.0
100.0
0.0
500.0
true
false
"" ""
PENS
"default" 1.0 0 -11881837 true "" "plot battery"

MONITOR
38
132
130
177
Drone battery
battery
17
1
11

INPUTBOX
1299
84
1371
144
yCoordBase
0.0
1
0
Number

SWITCH
33
237
136
270
restart
restart
0
1
-1000

MONITOR
19
302
89
347
Found fire
foundFire
17
1
11

MONITOR
102
301
177
346
Fire too big
fireTooBig
17
1
11

MONITOR
35
359
160
404
Average battery life
avgBatteryLife
17
1
11

MONITOR
1291
188
1377
233
random xFire
randXFire
17
1
11

MONITOR
1385
188
1471
233
random yFire
randYFire
17
1
11

MONITOR
25
425
155
470
found fire coord
ffc
17
1
11

MONITOR
16
491
160
536
Fire too big coord
ftbc
17
1
11

@#$#@#$#@
## WHAT IS IT?

A real time simulation of a drone detecting a fire in a chosen open space

## HOW IT WORKS

The drone scouts the area or goes to certain locations (chosen by the user) until it finds a fire

## HOW TO USE IT

1. Use the switch "restart" to make the simulation restart automatically and run until it is stopped or make it run only once. If the switch is set to on, the simulation will run until stopped. Otherwise, the simulation will end when the fire is detected or when it is stopped in case no fire was detected.

2. Press setup to set the environment.

3. Press run to start the simulatiom.

## CHOOSING A LOCATION THE DRONE HAS TO VISIT

1. Right click on one of the patches 

2. Click 'inspect patch'

3. In the bar below type "set xDestination pxcor" and press ENTER (this sends the x-coordinate of that patch to the drone)

4. In the same bar type "set yDestination pycor" and press ENTER (this sends the y-coordinate of that patch to the drone)

This location will be added to the list "destinations", which is printed in the command center. 

Repeat the steps above to add as many locations as desired

## THINGS TO NOTICE

There is a graph showing the battery of the drone over time.

If the "restart" switch is turned on, the number of found fires or fires that were not found in time is showed on 2 monitors. The coordinates of the fire are shown on 2 other monitors, one for when the fire was found, the other for when the fire was not found in time 

## THINGS TO TRY

Choose certain locations, as described in section "CHOOSING LOCATION THE DRONE HAS TO VISIT" to see the drone going to the locations. The list "destinations" which has the x,y-coordinates of the chosen locations as well as the distance from those locations to the drone are printed, so it can be noticed how locations are added and how they are ordered in ascending order by the distance.

Decrease the battery of the drone to see how it goes to the base and charges in time.

## EXTENDING THE MODEL

Choose a larger area with more drones scouting it. Also one could experiment with starting a simulation where the drone has already scouted for a certain amount time. In other words, the drone does not start with a full battery and its starting location is not on the base.

## NETLOGO FEATURES

The simulation uses the function "import-pcolors" which prints the chosen area to each patch; this is a key element to the simulation.


## CREDITS AND REFERENCES

The image used is a print screen of the sattelite image of the National Park Loonse en Drunense Duinen from google maps.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.1.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
